from swarm import Rexman as PC
from worlds import LayScape as World

class Space:

	def __init__(self, cs):
		self._pulse = 0 # Heartbeat start
		self._g = [0., 0.03] # Gravity / Wind
		self._r = [0.95, 0.999] # Resistance
		self._a = [0.05, 0.05] # Acceleration
		self._avatar = PC()
		self._horde = [] # NPC Horde
		self._scenery = World()
		self.start(cs)

	def start(self, cs):
		self._avatar.loc = [cs[0]/2 - self._avatar.size[0]/2, cs[1]/2 - self._avatar.size[1]/2] # Location
		self._avatar.vel = [0.,0.] # Velocity
		self._bnd = [cs[0] - self._avatar.size[0], cs[1] - self._avatar.size[1]] # Boundaries
		self._scenery.static('milky').xpm('chaos')

	def tick(self, act=[]):
		self._pulse += 1
		ac = self._avatar.act(act)
		self._physics(act, ac)
		return (self._avatar, self._horde, self._scenery, self._pulse)

	def _physics(self, act, ac):
		av = self._avatar # shortcut
		dimz = (0,1) # Dimensions

		# Map bounds check
		for i in dimz:
			av.vel[i]*=self._r[i]
			av.vel[i]+=self._g[i]
			av.loc[i]+=av.vel[i]
			if av.loc[i] > self._bnd[i] or av.loc[i] < 0:
				av.vel[i]*=-0.8
				for j in dimz:
					if j != i: av.vel[j]*=0.9
				if av.loc[i] > self._bnd[i]: av.loc[i]=self._bnd[i]-(av.loc[i]-self._bnd[i])
				if av.loc[i] < 0: av.loc[i]=-av.loc[i]
			if ac: av.vel[i] += ac[i] * self._a[i]
		bm, pt, av.alt = self._scenery.bump(av.loc, av.size) # Bump map, Penetration, Altitude

		# Landing mechanics
		if av.alt == None: av.alt = self._bnd[1] - av.loc[1]
		elif av.alt <= 0 and ((not ac or not ac[1]) or av.td == None):
			av.vel[1] = 0
			av.loc[1]-=pt[1][1]
			av.alt = 0
			if av.td == None: av.td = True # for touchdown w/o bump
			elif av.td: av.td = False

		# Bumpmap parsing
		bh=bv=0 # H/V Bump
		for i in bm:
			if i is 'l' and av.vel[0] < 0: bh-=1
			elif i is 'r' and av.vel[0] > 0: bh+=1
			elif i is 'u' and av.vel[1] < 0: bv+=1
			elif i is 'd' and av.vel[1] > 0: bv-=1
		if bv:
			av.vel[1]*=-0.2
			av.loc[1]+=(bv>0)*pt[0][1]-(bv<0)*pt[1][1]
		if bh and not av.td:
			if av.td == False: av.vel[0]*=-0.1
			elif not bv: av.vel[0]*=-0.2
			else: av.vel[0]=0
			av.loc[0]+=(bh>0)*pt[0][0]-(bh<0)*pt[1][0]
