import os, re
from toolz import Mx
from graphics import *

class LayScape:
	_img_dir = 'E:\\Others\\freed0m'
	_img_bg = 'bg_%s.png'
	_img_tile = 't_%s.png'
	_img_map = 'map_%s.xpm'
	_cache = {}

	img = None
	size = (240,320)
	cs = 'RGB16'

	_xpm_flags = {
		'h': FLIP_LEFT_RIGHT,
		'v': FLIP_TOP_BOTTOM,
		'1': ROTATE_180,
		'9': ROTATE_90,
		'0': ROTATE_270
	}
	_bg=_xpm_map=_xpm_mask=_xpm_scape=None

	def _compose(self):
		self.img = Image.new(self.size, self.cs)
		if self._bg: self.img.blit(self._bg)
		if self._xpm_map: self.img.blit(self._xpm_map, mask=self._xpm_mask)
		return self

	def static(self, name):
		cc = eval(self.__class__.__name__)
		id = 'bg_'+name
		try: self._bg = cc._cache[id]
		except KeyError: cc._cache[id] = self._bg = Image.open(os.path.join(self._img_dir, self._img_bg%name))
		return self._compose()

	def xpm(self, name):
		cc = eval(self.__class__.__name__)
		id = 'map_'+name
		try: self._xpm_map, self._xpm_mask, self._xpm_scape = cc._cache[id]
		except KeyError:
			file = open(os.path.join(self._img_dir, self._img_map%name)).readlines()
			xpm_attrz = re.compile('^\s*"(\d+) (\d+) (\d+) (\d+)",?\s*$')
			xpm_linez = xpm_tilez = False
			for line in file:
				if xpm_attrz:
					try:
						xpm_colz, xpm_line, xpm_tile = map(lambda x: int(x), xpm_attrz.match(line).group(1,2,3))
						dict, rmap, xpm_attrz = {}, {}, False
						xpm_tilez = re.compile('^\s*"(.)\s+(f\s+(\S+)\s+)?(s\s+(\S+)\s+)?(c\s+(\S+))?",?\s*$')
						xpm_linez = xpm_line
						self.res = self.size[1] / xpm_linez
						self._xpm_map = Image.new(self.size, self.cs)
						self._xpm_mask = Image.new(self.size, '1')
						self._xpm_map.clear(0)
						self._xpm_mask.clear(0)
						self._xpm_scape = Mx(xpm_colz, xpm_linez)
					except AttributeError: pass
				elif xpm_tile:
					try:
						code, flags, sym, color = xpm_tilez.match(line).group(1,3,5,7)
						xpm_tile -= 1
						if color == 'None': color = None
						else: color = eval('0x'+color[1:])
						dict[ord(code)] = (flags, sym, color)
					except AttributeError: pass
				elif xpm_line:
					line = line[line.find('"')+1:line.rfind('"')]
					for i in range(len(line)):
						try:
							flags, sym, color = dict[ord(line[i])]
							coords = (i * self.res, (xpm_linez-xpm_line) * self.res,
								i * self.res + self.res, (xpm_linez-xpm_line) * self.res + self.res)
							if not sym:
								if color:
									self._xpm_map.rectangle(coords, outline=color, fill=color)
									self._xpm_mask.rectangle(coords, outline=0xffffff, fill=0xffffff)
									self._xpm_scape[i, xpm_linez-xpm_line] = True
								else:
									self._xpm_scape[i, xpm_linez-xpm_line] = None
							else:
								tid = 't_'+sym
								try: tile = cc._cache[tid]
								except KeyError:
									tile = Image.new((self.res, self.res), '1')
									tile.load(os.path.join(self._img_dir, self._img_tile%sym))
									cc._cache[tid] = tile
								self._xpm_scape[i, xpm_linez-xpm_line] = True
								if flags:
									for i in flags:
										try: tile = tile.transpose(self._xpm_flags[i])
										except KeyError: pass # TODO: Add support for drawn tiles w/ mask here
								if color:
									self._xpm_map.rectangle(coords, outline=color, fill=color)
									self._xpm_mask.blit(tile, target=coords[0:2])
								else:
									self._xpm_map.blit(tile, target=coords[0:2])
									self._xpm_mask.rectangle(coords, outline=0xffffff, fill=0xffffff)
						except KeyError: pass
					xpm_line -= 1
			cc._cache[id] = (self._xpm_map, self._xpm_mask, self._xpm_scape)
		return self._compose()

	def bump(self, loc, size):
		dimz = (0,1) # Dimensions
		gmap = [(0,0),(1,0),(0,1),(1,1)]
		corners = [[loc[i]+corner[i]*size[i] for i in dimz] for corner in gmap]
		va, vz = min(corners), max(corners)
		pt = (self.res - va[0] % self.res, self.res - va[1] % self.res), (vz[0] % self.res, vz[1] % self.res) # Corners' penetration depth
		ma, mz = [int(va[i] / self.res) for i in dimz], [int(vz[i] / self.res)+(int(round(vz[i])%self.res) != 0)-1 for i in dimz]
		bm = ''
		for y in range(ma[1], mz[1]+1):
			for x in range(ma[0], mz[0]+1):
				bump = self._xpm_scape[x,y]
				if bump:
					if x == ma[0]: bm += 'l'
					if x == mz[0]: bm += 'r'
					if y == ma[1]: bm += 'u'
					if y == mz[1]: bm += 'd'
		alt = None
		for y in range(mz[1], self._xpm_scape.size[1]):
			for x in range(ma[0], mz[0]+1):
				if self._xpm_scape[x, y]:
					alt = (y - mz[1] - 1)*self.res + (self.res - pt[1][1])*(round(pt[1][1]) != 0)
					break
			if alt != None: break
		return bm, pt, alt
